/* @(#) 23 1.13 8/26/03 src/export/include/audit-plugin.h, ldap.audit, aus52ldap 10:04:04 */
/*
 * COMPONENT_NAME: ldap.audit
 *
 * ABSTRACT:
 *
 * FUNCTIONS:
 *
 * ORIGINS: 202,27
 *
 * (C) COPYRIGHT International Business Machines Corp. 2000,2004
 * All Rights Reserved
 * Licensed Materials - Property of IBM
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
/* $A1= D98606.1 5D20 040101 KKA      Audit changes                  */
/* $A2= D98606.3 5D20 051501 KKA      Change au_admin to char        */
/* $B1= P9A10439 5D30 021803 BLK      New define for when no local user */
/*
 * CHANGE HISTORY:
 *
 * DATE        INITIALS       COMMENT
 * 06/11/2003  BLK            D79945 - DIGEST-MD5 support.
 * 08/25/03    KKA            D81993  reasoning defect -- out of bounds array #121
 */

/*
 *
 * Licensed Materials - Property of IBM
 * 5647-A01,5722-SS1
 * (C) Copyright IBM Corp. 2000,2004
 *
 */

#ifndef _AUDIT_PLUGIN_H
#define _AUDIT_PLUGIN_H

#include <ldap.h>

/* au_boolean defines a value of either    */
/* AU_FALSE or AU_TRUE                     */
typedef unsigned char au_boolean;

/* au_ldapdn defines a character in        */
/* local code page                         */ 
typedef char          au_ldapdn;

#define AU_FALSE 			0
#define AU_TRUE  			1

#define AU_VER_UNKNOWN			0
#define AU_VER_2			1
#define AU_VER_3			2

#define AU_AUTH_UNKNOWN			0
#define AU_AUTH_ANONYMOUS		1
#define AU_AUTH_UNAUTHENTICATED		2
#define AU_AUTH_AUTHENTICATED		3

#define AU_NULL_DN			"<*CN=NULLDN*>"

/* P9A10439 - added AU_NO_LOCAL_USER */
#ifdef __OS400__
#define AU_NO_LOCAL_USER		"*NONE     "
#else
#define AU_NO_LOCAL_USER		"<*CN=NULLDN*>"
#endif

// D98606.3 added Y,N, and U
#define AU_YES                          'Y'
#define AU_NO                           'N'
#define AU_UNKNOWN                      'U'      

/* The following lists the operation code for an     */
/* audit record.                                     */
/* EXTOP are for LDAP v3 extended operations.        */
#define AU_OP_UNKNOWN			0
#define AU_OP_BIND			1
#define AU_OP_UNBIND			2
#define AU_OP_SEARCH			3
#define AU_OP_ADD			4
#define AU_OP_MODIFY			5
#define AU_OP_DELETE			6
#define AU_OP_MODIFYDN			7
#define AU_OP_EXTOP			8
#define AU_OP_COMPARE                   9


#define AU_EXTOP_UNKNOWN		0
#define AU_EXTOP_EN_REGISTRATION	1
#define AU_EXTOP_EN_UNREGISTER		2
#define AU_EXTOP_GENERAL   3

/* --------------------------------------------
 * represents the data passed to audit plug-ins 
 * --------------------------------------------
 */
#define AU_DATA_CONFIG_INFO		0
#define AU_DATA_EVENT                   1
#define AU_DATA_RECORD			2

typedef struct _Audit_data {
	int		au_data_type;	/* Audit config info, audit    */
	   		             	/* event or audit record       */
	void * 		au_data;	/* pointer to audit data       */		
} Audit_data;


/* ----------------------------------------
 * represents the configuration information 
 * related to LDAP auditing 
 * D81993 
 * NOTE: NUM_OPTIONS is the number of au_boolean fields in the struct
 * increment the number if more operations are audited
 * ----------------------------------------
 */
#define NUM_OPTIONS              11
typedef struct _Audit_config_info {
  char *	au_config_auditLog;	/* path/name of the log file */ 
  au_boolean	au_config_audit;	/* Doing Auditing?           */
  au_boolean	au_config_auditFailedOPonly;/*audit failed OP only?  */
  au_boolean	au_config_auditBind;	/* audit bind?               */
  au_boolean	au_config_auditUnbind;	/* audit unbind?             */
  au_boolean	au_config_auditSearch;	/* audit search?             */
  au_boolean	au_config_auditAdd;	/* audit add?                */
  au_boolean	au_config_auditModify;	/* audit modify?             */
  au_boolean	au_config_auditDelete;	/* audit delete?             */
  au_boolean	au_config_auditModifyDN;/* audit modifyDN?           */
  au_boolean	au_config_auditExtOPEvent;/* audit extended OP:      */
                                        /* event notification        */
   au_boolean au_config_auditExtOP;    /* audit all extended OPs   */
   int au_config_version;  /*audit version*/

}
Audit_config_info;

typedef struct _Audit_config_info_a {
  char *	au_config_auditLog;	/* path/name of the log file */ 
  au_boolean	au_config_options[NUM_OPTIONS];	/* array of config options D81993 */
} Audit_config_info_a;


/* --------------------------------------------
 * represents an event related to LDAP auditing
 * --------------------------------------------
 */
#define AU_EVENT_AUDITING_STARTED	0
#define AU_EVENT_AUDITING_ENDED  	1
#define AU_EVENT_CONFIG_OPTION_CHANGED	2
#define AU_EVENT_SERVER_STARTED         3
#define AU_EVENT_SERVER_ENDED           4

typedef struct _Audit_event {
        time_t          au_event_time;   /* time when event occurred     */
	int   		au_event_time_ms;/* au_event_time's milliseconds */ 
        int             au_event_number; /* audit event number           */
        char *		au_event_message;/* message associated with event*/
					 /* the text will be in local    */ 	
					 /* code page                    */ 	
} Audit_event;  


/* --------------------------
 * represents an audit record
 * --------------------------
 */
typedef struct _Audit_record {
	time_t		au_end_time;	/* time when operation processing is done */ 
	int   		au_end_time_ms;	/* au_end_time's milliseconds             */ 
	time_t		au_start_time;	/* time when operation processing started */ 
	int   		au_start_time_ms;/* au_start_time's milliseconds          */ 
	int		au_version;	/* LDAP version number                    */ 	
	au_boolean	au_ssl;		/* ssl connection?                        */
	int		au_auth;	/* anonymous or unauthenticated?          */
	int		au_connid;	/* connection ID                          */
	char *   	au_clientIp;  	/* client IP address                      */
	int             au_clientPort;	/* client port number                     */
	int		au_operation;	/* LDAP operation                         */
	int		au_extOP;	/* extended operation if au_operation is  */
					/* AU_OP_EXTOP                            */ 
	au_ldapdn *    	au_dn;        	/* bind dn                                */
	char *     	au_transID;	/* transaction ID                         */
	int		au_rc;       	/* result or status (LDAP error code)     */
	LDAPControl **	au_control;	/* pointer to control data, null if none. */
					/* This is a pointer to an array of ptrs  */
					/* which point to the LDAP control        */
					/* structures listed below.               */     
	void *		au_specific;	/* pointer to operation specific data,    */
	                	   	/* null if none. Use au_operation to      */
					/* determine which structure defined below*/
					/* to map over the data passed.           */
   char * au_local_user;  /* local user identify if known */
   char au_admin;        /* Y=administrator               */
                         /* N=not the administrator       */
                         /* U=unknown                     */
   au_boolean au_tls;  /* tls connection?  */
}
Audit_record;

/* == operation specific data ============================================*/

/* ---------------------------------
 * structure for Bind operation data
 * ---------------------------------
 */
#define AU_AUTH_CHOICE_UNKNOWN		0
#define AU_AUTH_CHOICE_SIMPLE		1
#define AU_AUTH_CHOICE_KRBV42LDAP	2
#define AU_AUTH_CHOICE_KRBV42DSA 	3
#define AU_AUTH_CHOICE_SASL		4

typedef struct _Audit_data_bind {
	au_ldapdn *	au_bind_name;		/* bind dn                 */
	int		au_bind_auth_choice;	/* authentication choice   */
	char *		au_bind_auth_mechanism;	/* authentication mechanism*/
   /* d79945 - new fields for DIGEST-MD5 bind auditing */
   char      *au_bind_digest_username;  /* username supplied on digest-md5 bind */
   char      *au_bind_digest_authzid;   /* authzid supplied on digest-md5 bind */
   au_ldapdn *au_bind_digest_mapped_dn; /* dn that username or authzid mapped to */
   /* d79945 - end of new fields for DIGEST-MD5 bind auditing. */
} Audit_data_bind;

/* --------------------------------------------------------------
 * there is no need to define structure for Unbind operation data
 * --------------------------------------------------------------
 */

/* -----------------------------------
 * structure for Search operation data
 * -----------------------------------
 */
#define AU_SCOPE_UNKNOWN      0
#define AU_SCOPE_BASE         1
#define AU_SCOPE_ONELEVEL     2
#define AU_SCOPE_SUBTREE      3

#define AU_DEREF_UNKNOWN      0
#define AU_DEREF_NEVER        1
#define AU_DEREF_SEARCHING    2
#define AU_DEREF_FINDING      3
#define AU_DEREF_ALWAYS       4

typedef struct _Audit_data_search {
	au_ldapdn *	au_search_baseObject;	/* base object entry      */ 
	int		au_search_scope;	/* scope of search        */ 
	int		au_search_derefAliases;	/* how alias is handled?  */ 
	au_boolean	au_search_typesOnly;	/* just return attr types?*/ 
	char *		au_search_filter;	/* search filter string   */ 
	char * 		au_search_attributes;	/* attributes to return,  */ 
	      		                  	/* attributes are listed  */ 
	      		                  	/* one by one, and        */ 
	      		                  	/* separated by comma (,) */ 
} Audit_data_search;

/* --------------------------------
 * structure for Add operation data
 * --------------------------------
 */
typedef struct _Audit_data_add {
	au_ldapdn *	au_add_entry;		/* entry to be added      */ 
	char *		au_add_attributes;	/* attributes to add,     */ 
	      		                  	/* attributes are listed  */ 
	      		                  	/* one by one, and        */ 
	      		                  	/* separated by comma (,) */ 
} Audit_data_add;

/* -----------------------------------
 * structure for Modify operation data
 * -----------------------------------
 */
typedef struct _Audit_data_modify {
	au_ldapdn *	au_modify_object;	/* object to modify       */ 
	LDAPMod **	au_modify_modification;	/* changes to be made,    */ 
	      		                  	/* LDAPMod is from ldap.h */ 
} Audit_data_modify;

#define AU_MOD_UNKNOWN        0
#define AU_MOD_ADD            1
#define AU_MOD_DELETE         2
#define AU_MOD_REPLACE        3


/* -----------------------------------
 * structure for Delete operation data
 * -----------------------------------
 */
typedef struct _Audit_data_delete {
	au_ldapdn * au_delete_entry;		/* entry to be deleted    */ 
} Audit_data_delete;

/* -------------------------------------
 * structure for ModifyDN operation data
 * -------------------------------------
 */
typedef struct _Audit_data_modifyDN {
	au_ldapdn *	au_modifyDN_entry; 	/* entry to be changed    */ 
	au_ldapdn *	au_modifyDN_newrdn;	/* new rdn                */ 
	au_boolean	au_modifyDN_deleteoldrdn;/* delete old rdn?       */ 
	au_ldapdn *	au_modifyDN_newSuperior;/* new immediate superior */ 
	           	                        /* NULL if none specified */
} Audit_data_modifyDN;


/* ----------------------------------------
 * structure for LDAP Event Notification
 * extended operation: Registration Request 
 * ----------------------------------------
 */

/* The following lists the event IDs for Event         */
/* Notification                                        */
#define AU_EXTOP_EN_ID_UNKNOWN			0
#define AU_EXTOP_EN_ID_LDAP_CHANGE		1


/* The following lists the changeTypes associated      */
/* with the LDAP Event Notification extended operation:*/
/* Registration request                                */ 
#define AU_EXTOP_EN_CHANGE_UNKNOWN	0
#define AU_EXTOP_EN_CHANGE_ADD		1
#define AU_EXTOP_EN_CHANGE_DELETE	2
#define AU_EXTOP_EN_CHANGE_MODIFY	3
#define AU_EXTOP_EN_CHANGE_MODDN	4

typedef struct _Audit_data_extOP_EN_registratoin {
	int		au_extOP_EN_reg_eventID;	/* event ID         */ 
	au_ldapdn *	au_extOP_EN_reg_baseObject;	/* base object      */ 
	int		au_extOP_EN_reg_scope;		/* scope            */ 
	char *		au_extOP_EN_reg_type;		/* type,            */ 
	      		                  		/* type are listed  */ 
	      		                  		/* one by one, and  */ 
	      		                  		/* separated by     */ 
	      		                  		/* comma (,)        */ 

} Audit_data_extOP_EN_registration;


/* ----------------------------------------
 * structure for LDAP Event Notification
 * extended operation: Unregister Request 
 * ----------------------------------------
 */

typedef struct _Audit_data_extOP_EN_unregister {
	char * 		au_extOP_EN_unreg_ID;		/* registration ID */ 
							/* to unregister   */ 

}
Audit_data_extOP_EN_unregister;

/* ----------------------------------------
 * structure for LDAP Extended operation 
 * ----------------------------------------
 */

typedef struct _Audit_data_extOP {
   char * au_extOP_oid;  /* extended op oid */
   char * au_extOP_info;       /* additional info about extended op*/

}
Audit_data_extOP;

/* --------------------------------
 * structure for Compare operation data
 * --------------------------------
 */
typedef struct _Audit_data_compare {
        au_ldapdn *     au_compare_object;
        char *          au_compare_attribute;
} Audit_data_compare;

#endif /* _AUDIT_PLUGIN_H */
